/*** webpack.config.js ***/
const path = require('path');

const ModuleFederationPlugin = require("webpack").container
  .ModuleFederationPlugin;

module.exports = {
    entry: path.join(__dirname, "index.js"),
    output: {
        path: path.join(__dirname, "/build"),
        filename: "bootstrap-core.bundle.js"
    },
    module: {
        rules: [

            {
                test: /\.js$|jsx/,
                // test: /\.js$/,
                use: "babel-loader",
                // exclude: /node_modules/
            },
            {
                test: /\.(s(a|c)ss)$/,
                use: ['css-loader', 'sass-loader'],
                // exclude: /node_modules/
            },
            {
                test: /\.(css)$/,
                use: ['css-loader'],
                // exclude: /node_modules/
            },

            {
                test: /\.tsx?$/,
                use: 'ts-loader',
                // exclude: /node_modules/,
            },

            {
                test: /\.(png|jpe?g|gif)$/i,
                use: [
                    {
                        loader: 'file-loader',
                    },
                ],
                // exclude: /node_modules/
            },

        ]
    },
    resolve: {
        extensions: [".js", ".tsx"]
    },
    devServer: {
        port: 5003
    },
    plugins: [
        new ModuleFederationPlugin({
            name: "application-loader",
            shared: {
                react: {
                    singleton: true,
                    eager: true,
                    requiredVersion: packageJson.dependencies.react
                },
                "react-dom": {
                    singleton: true,
                    eager: true,
                    requiredVersion: packageJson.dependencies['react-dom']
                },
                // "react-router": {
                //   singleton: true,
                //   eager: true,
                //   requiredVersion: packageJson.dependencies['react-router']
                // },
                "react-router-dom": {
                    singleton: true,
                    eager: true,
                    requiredVersion: packageJson.dependencies['react-router-dom']
                },
                // "react-i18next": {
                //   singleton: true,
                //   eager: true,
                //   requiredVersion: packageJson.dependencies['react-i18next']
                // }
            },
        }),

        // Removes/cleans build folders and unused assets when rebuilding
        new CleanWebpackPlugin(),

        // Copies files from target to destination folder
        new CopyWebpackPlugin({
            patterns: [
                {
                    from: paths.public,
                    to: 'assets',
                    globOptions: {
                        ignore: ['*.DS_Store'],
                    },
                    noErrorOnMissing: true,
                },
            ],
        }),

        // Generates an HTML file from a template
        // Generates deprecation warning: https://github.com/jantimon/html-webpack-plugin/issues/1501
        new HtmlWebpackPlugin({
            title: 'Application Loader',
            // favicon: paths.src + '/images/favicon.png',
            template: paths.src + '/index.html', // template file
            filename: 'index.html', // output file
        }),

        // ESLint configuration
        new ESLintPlugin({
            files: ['./config/eslint.json'],
            formatter: 'table',
        }),

        // Prettier configuration
        new PrettierPlugin(),

        new WebpackManifestPlugin({
            fileName: 'module-manifest.json'
        }),
        new WebpackBundleAnalyzer(),

        // new webpack.DefinePlugin({
        //   'process.env.NODE_ENV': JSON.stringify('development'),

        //   'process': JSON.stringify('development'),

        // }),

        // new webpack.ProvidePlugin({
        //   process: 'process/browser',
        // }),


    ],


  // Determine how modules within the project are treated
  module: {
    rules: [
      // JavaScript: Use Babel to transpile JavaScript files
      {
        test: /\.js$/,
        use: {
          loader: 'babel-loader',
          options: {
            customize: require.resolve(
              'babel-preset-react-app/webpack-overrides'
            ),
            presets: [
              [
                require.resolve('babel-preset-react-app'),
                {
                  runtime: 'classic',
                },
              ],
            ],
          }
        }

      },

      // Images: Copy image files to build folder
      { test: /\.(?:ico|gif|png|jpg|jpeg)$/i, type: 'asset/resource' },

      // Fonts and SVGs: Inline files
      { test: /\.(woff(2)?|eot|ttf|otf|svg|)$/, type: 'asset/inline' },
      {
        test: /\.woff(2)?(\?v=[0-9]\.[0-9]\.[0-9])?$/,
        include: path.resolve(__dirname, './node_modules/bootstrap-icons/font/fonts'),
        use: {
          loader: 'file-loader',
          options: {
            name: '[name].[ext]',
            outputPath: 'webfonts',
            publicPath: '../webfonts',
          },
        }
      }
    ],
  },

  resolve: {
    modules: [paths.src, 'node_modules'],
    extensions: ['.js', '.jsx', '.json'],
    alias: {
      '@': paths.src,
    },
  },
};